## Bootstrap Confidence Bands
## non sparse

source("vem_dynppsbm.R")
source("output.R")

bootstrapCI <- function(sol,Time,R,alpha=0.05,nbcores=1,l_part=5,n_perturb=10,perc_perturb=.2,directed,filename=NULL){
  Dmax <- ncol(sol$loglambda.ql)
  Q <- nrow(sol$tau)
  n <- ncol(sol$tau)
  N_Q <- if (directed)  Q^2 else Q*(Q+1)/2
  boot.sol <- array(NA,dim=c(R,N_Q,Dmax)) # storage of loglambda.ql values
  all.intens <- exp(sol$loglambda.ql)
  all.intens[sol$loglambda.ql==0] <- 0  
  prop.groups=rowMeans(sol$tau)
  for (i in 1:R){
    cat('R:',i,'\n')
    data <- generat.const.dynppSBM(all.intens,Time,n,prop.groups,directed)
    Nijk <- statistics(data$data,n,Dmax,directed)
    obs <- list(Nijk=Nijk,Time=Time)
    sol <- dynppSBM(obs,n,Qmin=Q,Qmax=Q,directed,method='hist',cores=nbcores,l_part,n_perturb,perc_perturb)[[1]]
    # label switching
    boot.sol[i,,] <- permut.lambda(sol$loglambda.ql,data$z,sol$tau,directed)
    if (!is.null(filename))
      save(boot.sol,file=filename)
  }
  
  CIlim <- boot.CIlimits(boot.sol,alpha)
  return(list(CI.limits=CIlim, boot.sol=boot.sol))
}

#############
# generate data under dynppsbm with piecewise constant intensities
#############
# intens: matrix with piecewise constant intensities alpha^(q,l) (each row gives the contstants of the piecewise constant intensity for a group pair (q,l))
# prop.groups: vector of group proportions, should be of length Q
# if directed=TRUE then intens should be of length Q^2 and if directed =FALSE then length Q*(Q+1)/2

generat.const.dynppSBM <- function(intens,Time,n,prop.groups,directed=TRUE){
  Q <- length(prop.groups)
  
  # test coherence
  N_Q <- if (directed)  Q^2 else Q*(Q+1)/2
  if (nrow(intens)!=N_Q) stop("not a correct number of intensities")
    
  # Draw hidden groups
  z <- rmultinom(n, 1, prop.groups)
  group <- apply(z, 2, which.max)
  
  # Generate the process
  time.seq <- NULL
  type.seq <- NULL
  vec_i <- if (directed)  1:n else 1:(n-1)
  for (i in vec_i){
    vec_j <- if (directed) (1:n)[-i] else (i+1):n
    for (j in vec_j){
      type.ql <- indice(group[i],group[j],Q,directed)
      proc <- generat.const.intens(intens[type.ql,],Time)
      if (length(proc)>0){
        time.seq <- c(time.seq,proc)  	# not ordered 	
        type.seq <- c(type.seq,rep(which.dyads(i,j,n,directed),length(proc)))  			
      }
    }
  }
  ordre <- order(time.seq)
  time.seq <- time.seq[ordre]
  type.seq <- type.seq[ordre]
  data <-	list(time.seq=time.seq,type.seq=type.seq,Time=Time)
  
  return(list(data=data,z=z,intens=intens))
}


#########
# generate realizations of a PP of with piecewise constant intensity 
# intens is a vector with the constants of the intensity (defined on a regular partition of interval [0,Time])
#########
generat.const.intens <- function(intens,Time){
  K <- length(intens)
  long.subint <- Time/K
  area <- intens*long.subint
  nb.points.interv <- rpois(rep(1,K),area)
  points <- NULL
  for (k in 1:K)
    if (nb.points.interv[k]>0)
      points <- c(points,runif(nb.points.interv[k],(k-1)*long.subint,k*long.subint))
  return(points)	
}

#########
# compute confidence intervals for all pair of groups (q,l)
#########
boot.CIlimits <- function(boot.sol,alpha=0.05){
  dim.sol <- dim(boot.sol)
  R <- dim.sol[1]
  N_Q <- dim.sol[2]
  Dmax <- dim.sol[3]
  CIlim <- array(NA,dim=c(N_Q,3,Dmax))
  for (ql in 1:N_Q)
    CIlim[ql,,] <- apply(boot.sol[,ql,],2,function(x){quantile(x,probs=c(alpha/2,1-alpha/2,.5))})
  CIlim <- exp(CIlim)
  return(CIlim)
}


