################################
################################
#### LONDON BIKE DATASET ANALYSIS
################################
################################


################################
#### Preprocessing of the data
################################

rm(list=ls())

#########
# Extract the data
#########
# extraction of February dataset 
data<-read.table("London_cycles_Journey_Data_Extract_01Feb-29Feb_12.csv",header=T,sep=",")

#############
## keep only data needed
#############
# Filter the data
# Remove trips less than 1 min. 
data <- data[-which(data$Duration<60),]
# Remove trips longer than 3 hours 
data <- data[-which(data$Duration>3*60*60),]
# Keep EndStation.Id, EndStation.Name, Start.Date, StartStation.Id, StartStation.Name
data <-data[,5:9]
# throw away the loops
data <- data[-(which(data[,1]==data[,4])),]
# Throw away un-returned bikes
data <- data[-(which(data[,1]==0)),]

Nb.events <- length(data[,1])
# reduced to 456 412 interactions

## Stations list (character)
Uni.Station <- unique(c(data$EndStation.Id,data$StartStation.Id))
n <- length(Uni.Station)
# Only 457 stations in the dataset
N <- n*(n-1)

#############
## GPS locations
#############
locations <- read.table('London_cycles_locations_stations.txt',header=T,sep=";")
# length(unique(locations$Id))
# 748 stations in total  

inter<-intersect(locations$Id,Uni.Station)
# length(inter)
# Only 449 stations among the 457 we use are geolocalised. 

######
# Transform date in time expressed in seconds ; 0 = 01/02/2012 00:00:00 
######
start.date <- unlist(strsplit(as.character(data$Start.Date), split="/"))
start.day <- as.numeric(start.date[(1:Nb.events)*3-2])
start.time <- unlist(strsplit(as.character(start.date[(1:Nb.events)*3]),split=" "))[(1:Nb.events)*2]
start.time <- as.numeric(unlist(strsplit(as.character(start.time),split=":")))

time.seq <- (start.day-1)*3600*24 + start.time[(1:Nb.events)*3-2]*3600 + start.time[(1:Nb.events)*3-1]*60 +start.time[(1:Nb.events)*3]
time.seq <- time.seq -min(time.seq)
data$Start.Date <- time.seq


####################
# Save CLEANED DATA
####################
# save(file="London_cycles_data_Feb2012_cleaned_directed.Rdata",data,n,N,Nb.events,Uni.Station,locations)


################################################################################
################################################################################
######## Extract DAYS from the data ############################################
################################################################################

####################
# Focus on just one day (1st or 2nd February)
####################

data.day1<-data[data$Start.Date<24*3600,]
data.day2<-data[(24*3600<data$Start.Date & data$Start.Date<2*24*3600),]

Nb.events.day1 <- length(data.day1[,1])
Nb.events.day2 <- length(data.day2[,1])

######
# Renumbering stations from 1 to n1 in day1 
toto <- c(data.day1$EndStation.Id,data.day1$StartStation.Id)
toto <- match(toto, unique(sort(toto)))
data.day1$EndStation.Number <-toto[1:length(data.day1$EndStation.Id)]
data.day1$StartStation.Number <- toto[(length(data.day1$EndStation.Id)+1):(length(data.day1$EndStation.Id)+length(data.day1$StartStation.Id))]
n1 <- length(unique(c(data.day1$EndStation.Id,data.day1$StartStation.Id)))
N1 <- n1*(n1-1)


# Transform pair of dyads (i,j) into type for DAY 1 using n1 value
# the (i,j) relation (i\neq j) is stored in variable with index (i-1)*(n-1)+j-(i<j) 
type.seq <- (data.day1$StartStation.Number-1)*(n1-1)+data.day1$EndStation.Number-(data.day1$StartStation.Number < data.day1$EndStation.Number)
# length(unique((type.seq)))
obs.day1 <- list(time.seq=data.day1$Start.Date,type.seq=type.seq,Time=24*3600)


######
# Renumbering stations from 1 to n2 in day2 
toto <- c(data.day2$EndStation.Id,data.day2$StartStation.Id)
toto <- match(toto, unique(sort(toto)))
data.day2$EndStation.Number <-toto[1:length(data.day2$EndStation.Id)]
data.day2$StartStation.Number <- toto[(length(data.day2$EndStation.Id)+1):(length(data.day2$EndStation.Id)+length(data.day2$StartStation.Id))]
n2 <- length(unique(c(data.day2$EndStation.Id,data.day2$StartStation.Id)))
N2<-n2*(n2-1)


# Transform pair of dyads (i,j) into type for DAY 2 using n2 value
# the (i,j) relation (i\neq j) is stored in variable with index (i-1)*(n-1)+j-(i<j) 
type.seq <- (data.day2$StartStation.Number-1)*(n2-1)+data.day2$EndStation.Number-(data.day2$StartStation.Number < data.day2$EndStation.Number)
# length(unique((type.seq)))
obs.day2 <- list(time.seq=data.day2$Start.Date-24*3600,type.seq=type.seq,Time=24*3600)


#############
# Construct point processes statistics for days 1,2 - with dyadics partitions 2^5 
#############
setwd("../code/")
source("auxiliary.R")
setwd("../data/")
Dmax <- 2^5  #32
obs.day1.dyad5 <- list(Time=obs.day1$Time,Nijk = statistics(obs.day1,n1,Dmax))
obs.day2.dyad5 <- list(Time=obs.day2$Time,Nijk = statistics(obs.day2,n2,Dmax))
#save(file="London_cycles_data_01Feb2012_directed_Dyadique-5.Rdata",data.day1,obs.day1,obs.day1.dyad5,n1,N1,Dmax,locations)
#save(file="London_cycles_data_02Feb2012_directed_Dyadique-5.Rdata",data.day2,obs.day2,obs.day2.dyad5,n2,N2,Dmax,locations)
###


################################
################################
################################
#### Dynppsbm analysis - directed (non sparse)
################################
setwd("../code/")
source("vem_dynppsbm.R")
source('output.R')
setwd("../data/")

#load("London_cycles_data_01Feb2012_directed_Dyadique-5.Rdata")
data <- obs.day1.dyad5
N <- dim(data$Nijk)[1]
n <- (1+sqrt(1+4*N))/2      # n = 415
n
Qmin <- 1
Qmax <- 10
nb.coeurs <- detectCores()

## WARNING ! THIS STEP TAKES SOME TIME !
# You can go directly to loading our results
res <- dynppSBM(data,n,Qmin=Qmin,Qmax=Qmax,directed=TRUE,sparse=FALSE,cores=nb.coeurs,l_part=4,n_perturb=20,perc_perturb=0.25,n_random=0,filename='vem_01Feb2012_Dmax-5_modelsel_1_10.Rdata')
#save(res,file='London_cycles_vem_01Feb2012_Dmax-5_modelsel_1_10.Rdata')

############
load("London_cycles_vem_01Feb2012_Dmax-5_modelsel_1_10.Rdata")
modelsel <- model.selection_Q(data,n,Qmin,Qmax,directed=TRUE,sparse=FALSE,sol.hist.sauv=res)
plot.modelselec.Q(modelsel)
modelsel$all.compl.log.likelihood
modelsel$all.ICL
modelsel$all.J

# Extract the best classification 
sol <- modelsel$sol.Qbest
q <- dim(sol$tau)[1]
map <- apply(sol$tau,2,which.max)


########
## DAY2 - Switch here to analyse day2
### 
#load("London_cycles_data_02Feb2012_directed_Dyadique-5.Rdata")
data <- obs.day2.dyad5
N <- dim(data$Nijk)[1]
n <- (1+sqrt(1+4*N))/2      # n=417
n
Qmin <- 1
Qmax <- 10
nb.coeurs <- detectCores()

## WARNING ! THIS STEP TAKES SOME TIME !
# You can go directly to loading our results
res <- dynppSBM(data,n,Qmin=Qmin,Qmax=Qmax,directed=TRUE,sparse=FALSE,cores=nb.coeurs,l_part=4,n_perturb=20,perc_perturb=0.25,n_random=0,filename='vem_02Feb2012_Dmax-5_modelsel_1_10.Rdata')
#save(res,file='London_cycles_vem_02Feb2012_Dmax-5_modelsel_1_10.Rdata')

###### 
load("London_cycles_vem_02Feb2012_Dmax-5_modelsel_1_10.Rdata")
modelsel <- model.selection_Q(data,n,Qmin,Qmax,directed=TRUE,sparse=FALSE,sol.hist.sauv=res)
plot.modelselec.Q(modelsel)
modelsel$all.compl.log.likelihood
modelsel$all.ICL


# Extract the best classification 
sol <- modelsel$sol.Qbest
q <- dim(sol$tau)[1]
map <- apply(sol$tau,2,which.max)

######################################################################################################
######### RESULTS ANALYSIS - days 1,2
######################################################################################################
### With q = best number of groups
q <- 6
# Stations per group
for (k in 1:q){
  print(sum(map==k))
}

# Small group - for day1 and 2 it's k=4
k<-4
ind<-which(map==k)
gpe <-c()
for (i in ind){
  gpe<-c(gpe,data.day1$EndStation.Id[which.max(data.day1$EndStation.Number==i)])	
}
# for (i in ind){
# gpe<-c(gpe,data.day2$EndStation.Id[which.max(data.day2$EndStation.Number==i)])
# }
gpe
# 14;Belgrove Street, Kings Cross;001011;51.52994371;-0.123616824
# 154;Waterloo Station 3, Waterloo;001072;51.50379168;-0.11282408
### 


######## Plot the clusters 
## GRAPHICAL LIBRARIES
#install.packages("OpenStreetMap")
library("OpenStreetMap")
#install.packages('maps')
library('maps')
#install.packages('ggplot2')
library('ggplot2')
#install.packages('sp')
library('sp')


## Geolocalisation
GPS<-list()
for  (k in 1:q){
  tata <- (1:n)[map==k]   
  GPS[[k]] <- matrix(NA,nrow=length(tata),ncol=2)
  j <- 1
  for (i in tata){
    ind <- locations$Id == data.day1$EndStation.Id[which.max(data.day1$EndStation.Number==i)]
    # ind <- locations$Id == data.day2$EndStation.Id[which.max(data.day2$EndStation.Number==i)]
    if (sum(ind)==1){ 
      GPS[[k]][j,] <- c(locations$long[ind],locations$lat[ind])
    }
    j <- j+1
  }
  GPS[[k]] <- na.omit(GPS[[k]])
  GPS[[k]] <- SpatialPoints(GPS[[k]])
  proj4string(GPS[[k]]) <- CRS("+proj=longlat +ellps=WGS84")
  GPS[[k]] <- spTransform(GPS[[k]],CRS("+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs"))
}

# Borders are set by hand
london.map <- openmap(c(51.54,-0.21), c(51.48,-0.06),type='osm')
#autoplot(london.map)

filename<-paste0("clusters_day1_q",q,".pdf")
# filename<-paste0("clusters_day2_q",q,".pdf")
pdf(filename,width = 6.2,height=4)
## Plot the map and the points
plot(london.map,raster=TRUE)
for (k in (q:1)){
  plot(GPS[[k]],add=TRUE,col=k,pch=k,lwd=2)
}
dev.off()


#### 
## Temporal profile of the small cluster  
####
Nijk <- obs.day1.dyad5$Nijk
# Nijk <- obs.day2.dyad5$Nijk
k<-4
ind<-which(map==k)
clean_hours <- function(time){
  hour <- floor(time)
  min <- (time -floor(time))*60
  res <- paste0(hour,":",min)
  res[min==0] <- paste0(hour[min==0])
  return(res)
}
hours <- (1:32)*data$Time/2^5 / 3600
c.hours <- clean_hours(hours)
# Belgrove St - King's Cross
i <- ind[1]
j<-(1:n)[-i]
dij <- data.frame(tim=factor(1:32,labels=c.hours),count=apply(Nijk[(i-1)*(n-1)+j-(i<j),],2,sum))
plot.out1 <- ggplot(data = dij, aes(x=tim,y =count)) + geom_bar(stat="identity") +
  theme(plot.title = element_text(hjust = 0.5),axis.text.x=element_text(angle=90,hjust=1,vjust=0.5,size=7)) +
  labs(x="Time",y="Outgoing counts",title="Kings Cross")
dji <-  data.frame(tim=factor(1:32,labels=c.hours),count=apply(Nijk[(j-1)*(n-1)+i-(j<i),],2,sum))
plot.in1 <- ggplot(data = dji, aes(x=tim,y =count)) + geom_bar(stat="identity") +
  theme(plot.title = element_text(hjust = 0.5),axis.text.x=element_text(angle=90,hjust=1,vjust=0.5,size=7)) +
  labs(x="Time",y="Incoming counts",title="Kings Cross")
# Waterloo 3
i <- ind[2]
j<-(1:n)[-i]
dij <- data.frame(tim=factor(1:32,labels=c.hours),count=apply(Nijk[(i-1)*(n-1)+j-(i<j),],2,sum))
plot.out2 <- ggplot(data = dij, aes(x=tim,y =count)) + geom_bar(stat="identity") +
  theme(plot.title = element_text(hjust = 0.5),axis.text.x=element_text(angle=90,hjust=1,vjust=0.5,size=7)) +
  labs(x="Time",y="Outgoing counts",title="Waterloo 3")
dji <-  data.frame(tim=factor(1:32,labels=c.hours),count=apply(Nijk[(j-1)*(n-1)+i-(j<i),],2,sum))
plot.in2 <- ggplot(data = dji, aes(x=tim,y =count)) + geom_bar(stat="identity") +
  theme(plot.title = element_text(hjust = 0.5) ,axis.text.x=element_text(angle=90,hjust=1,vjust=0.5,size=7)) +
  labs(x="Time",y="Incoming counts",title="Waterloo 3")
final.plot <- grid.arrange(plot.out1, plot.in1,plot.out2,plot.in2,ncol=2,nrow=2)
ggsave('Day1_temporal_profiles_gp4.pdf',final.plot,width = 6.5,height=6)


# Pics: 
max.out <- which.max(apply(Nijk[(i-1)*(n-1)+j-(i<j),],2,sum))
cat('Max Departures at',max.out*data$Time/2^5 / 3600, 'h') 
max.in <- which.max(apply(Nijk[(j-1)*(n-1)+i-(j<i),],2,sum))
cat('Max Arrivals at',max.in*data$Time/2^5 / 3600, 'h') 

##### Temporal profile of other clusters
# For instance Waterloo 1 does not have the same profile
i <- 364
j<-(1:n)[-i]
par(mfrow=(c(1,2))) 
barplot(Nijk[(i-1)*(n-1)+j-(i<j),], main="Waterloo 1 - N_{i.}") 
barplot(Nijk[(j-1)*(n-1)+i-(j<i),], main="Waterloo 1 - N_{.i}") 


######
### Look at the intensities for all groups 
###
q <- 6
D.values <- rep(NA,q^2)
T <- data$Time
for (k in 1:q)
  for (l in 1:q){
    D.values[indice(k,l,q,directed=T)] <- length(sol$loglambda.ql[indice(k,l,q,directed=T),])
  }	
D.values
# This is the sizes of the partitions that are used. Here we have nothing constant 


####
# Kernel estimator from groups estimated by histogram method
kernel <- hat.lambda.fn(obs.day1,sol$tau,q,n,directed=T)
# kernel <- hat.lambda.fn(obs.day2,sol$tau,q,n,directed=T)
T <- data$Time
y.max <- max(kernel) 
par(mfrow=c(3,2))
for (k in 1:q){
  # first part of plots
  for (l in 1:q){
  xtitle <-paste0("(",k,",",l,")")
  ind.ql <- indice(k,l,q,directed=TRUE)
  plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
  }
  readline("Press <return to continue") 
}



# PLots that are non almost null for day 1 
library(gridExtra)
k =3
l = 4
ind.ql <- indice(k,l,q,directed=TRUE)
p1 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
k=4
l=3
ind.ql <- indice(k,l,q,directed=TRUE)
p2 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
l=5
ind.ql <- indice(k,l,q,directed=TRUE)
p3 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
k=4
l=6
ind.ql <- indice(k,l,q,directed=TRUE)
p4 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
k=5
l=4
p5 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
l=5
p6 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
k=6
l=4
p7 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
l=5
p8 <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
  geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
final.plot<-grid.arrange(p1,p2,p3,p4,p5,p6,p7,p8,ncol=2,nrow=4)
# pdf("Day1_non_zero_intens_part1.pdf",width=6.2,height=5)
##pdf("Day1_non_zero_intens_part2.pdf",width=6.2,height=5)
ggsave('Day1_non_zero_intens.pdf',final.plot, width = 6.2,height=8)
#dev.off()

#######
# PLots that are non almost null for day 2 
pdf("Day2_non_zero_intens_part1.pdf",width=6.2,height=4)
par (mfrow=c(2,2))
k =2
l = 4
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
k=4
l=2
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
l=4
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
l=5
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
dev.off()
### pause here
pdf("Day2_non_zero_intens_part2.pdf",width=6.2,height=4)
par (mfrow=c(2,2))
k=4
l=6
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
k=5
l=4
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
l=5
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
k=6
l=4
xtitle <-paste0("(",k,",",l,")")
ind.ql <- indice(k,l,q,directed=TRUE)
plot(seq(0,T,by=T/1000),kernel[ind.ql,],type='l',ylim=c(0,y.max),col=2,lty=1,xlab=xtitle,ylab="")
dev.off()



################################
################################
################################
#### Dynppsbm analysis - directed AND sparse
################################

### Day 1
#setwd("../code/")
#source("vem_dynppsbm.R")
#source('output.R')
#setwd("../data/")

# load("London_cycles_data_01Feb2012_directed_Dyadique-5.Rdata")
data <- obs.day1.dyad5
N <- dim(data$Nijk)[1]
n <- (1+sqrt(1+4*N))/2      # n = 415
n
Qmin <- 1
Qmax <- 10
nb.coeurs <- detectCores()

## WARNING ! THIS STEP TAKES SOME TIME !
# You can go directly to loading our results
sol <- dynppSBM(data,n,Qmin=Qmin,Qmax=Qmax,directed=TRUE,sparse=T,cores=nb.coeurs,l_part=3,n_perturb=20,perc_perturb=0.25,n_random=0,filename='London_cycles_vem_01Feb2012_Dmax-5_modelsel_1_10_sparse.Rdata')
#save(sol,file='London_cycles_vem_01Feb2012_Dmax-5_modelsel_1_10_sparse.Rdata')


###
load("London_cycles_vem_01Feb2012_Dmax-5_modelsel_1_10_sparse.Rdata")
modelsel <- model.selection_Q(data,n,Qmin,Qmax,directed=TRUE,sparse=T,sol.hist.sauv=sol)
plot.modelselec.Q(modelsel)
modelsel$all.compl.log.likelihood
modelsel$all.ICL
modelsel$all.J
# best model obtained with Q=2

# Note that J is badly maximized - however 
for (q in 1:Qmax){
  map <- apply(sol[[q]]$tau,2,which.max)
  print(paste0("effective number of groups=" ,length(unique(map))))
}
# for Q in {2,3,4,5,6}, the best solutions have only 2 non-empty groups 


# analysis with q=2 groups 
q<- 2
sol2 <-sol[[q]]
map2 <- apply(sol2$tau,2,which.max)
for (k in 1:q){
  print(paste("number of stations in group k=",k,"is",sum(map2==k)))
}
# one very large group and one medium group 

## Geolocalisation
GPS<-list()
for  (k in 1:q){
  tata <- (1:n)[map2==k]
  GPS[[k]] <- matrix(NA,nrow=length(tata),ncol=2)
  j <- 1
  for (i in tata){
    ind <- locations$Id == data.day1$EndStation.Id[which.max(data.day1$EndStation.Number==i)]
    # ind <- locations$Id == data.day2$EndStation.Id[which.max(data.day2$EndStation.Number==i)]
    if (sum(ind)==1){
      GPS[[k]][j,] <- c(locations$long[ind],locations$lat[ind])
    }
    j <- j+1
  }
  GPS[[k]] <- na.omit(GPS[[k]])
  GPS[[k]] <- SpatialPoints(GPS[[k]])
  proj4string(GPS[[k]]) <- CRS("+proj=longlat +ellps=WGS84")
  GPS[[k]] <- spTransform(GPS[[k]],CRS("+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs"))
}

# Borders are set by hand
london.map <- openmap(c(51.54,-0.21), c(51.48,-0.06),type='osm')
#autoplot(london.map)

## Plot the map and the points
filename<-paste0("clusters_day1_q",q,"_sparse.pdf")
pdf(filename,width = 6.2,height=4)
par(mfrow=c(1,1))
plot(london.map,raster=TRUE)
for (k in (1:q)){
  plot(GPS[[k]],add=TRUE,col=k,pch=k,lwd=2)
}
dev.off()


kernel <- hat.lambda.fn(obs.day1,sol2$tau,q,n,directed=T)
T <- data$Time
plot.list=list()
for (k in 1:q){
  for (l in 1:q){
    ind.ql <- indice(k,l,q,directed=TRUE)
    plot.list[[ind.ql]] <- ggplot(data=data.frame(Time=seq(0,T,by=T/1000),kernel=kernel[ind.ql,]), aes(x=Time,y=kernel,group=1))+ 
      geom_line(linetype="solid") +labs(title =paste0("(q,l)=(",k,",",l,")"),y="Intensity")+theme(plot.title = element_text(hjust = 0.5))
  }
}
last.plot <- do.call("grid.arrange",c(plot.list, ncol=2,nrow=2))
ggsave("day1_intensities_sparse.pdf",plot=last.plot, width = 6.2,height=5)


### modes for interaction (2,2) (ind.ql=4)
x <- seq(0,T,by=T/1000)
mode1 <- x[which.max(kernel[4,])]/60/60
mode1
# at 8:30 am
dec <- which.max(x>60000)
mode2 <- x[dec + which.max(kernel[4,(x>60000)])]/60/60
mode2
# at 5:50pm 
dec2 <- which.max(x>40000)
mode3 <- x[dec2 + which.max(kernel[4,((x>40000)&(x<55000))])]/60/60
mode3

### mode for (q,l)=(1,2) ind.ql=2
mod.12 <-  x[which.max(kernel[2,])]/60/60
mod.12
# at 8:20 am
### mode for (q,l)=(2,1) ind.ql=3
mod.21  <-  x[which.max(kernel[3,])]/60/60
mod.21
# at 17:50

# sparse parameters
matrix(sol2$rho,2,2)
matrix(sol2$beta,2,2)



###################################################
###################################################
### Comparisons with Dynsbm (Matias-Miele)
###################################################
##################################

##### Day 1
# rm(list=ls())# rm(list=ls())
# load("London_cycles_data_01Feb2012_directed_Dyadique-5.Rdata")
#setwd("../code/")
#source("vem_dynppsbm.R")
#setwd("../data/")
Dmax <- 2^10  #1024
obs.day1.dyad10 <- list(Time=obs.day1$Time,Nijk = statistics(obs.day1,n1,Dmax))
# if working with day2 - Beware starting time
# data.day2$Start.Date <- data.day2$Start.Date -24*3600
# range(data.day2$Start.Date)

# Let's construct T=24 adjacency matrices with size 24 x n1 xn1 
T<- 24
Y<-array(0,dim=c(T,n1,n1))
for (t in 1:T){
  i.index <-data.day1$StartStation.Number[((t-1)*3600 < data.day1$Start.Date)&(data.day1$Start.Date <= t*3600)]
  j.index<- data.day1$EndStation.Number[((t-1)*3600 < data.day1$Start.Date)&(data.day1$Start.Date <= t*3600)]
  for (s in 1:length(i.index)){
    # weighted version
    #Y[t,i.index[s],j.index[s]] <- Y[t,i.index[s],j.index[s]]+1
    # binary version
    Y[t,i.index[s],j.index[s]] <- +1
  }
}

##############
#### Dynsbm run
#install.packages("dynsbm")
library(dynsbm)
nb.coeurs <- detectCores()
P <- matrix(1,nrow=n1,ncol=T)
res_dynsbm <- select.dynsbm(Y, present=P, Qmin=1, Qmax=10, edge.type="binary", directed=T, nb.cores=nb.coeurs,  iter.max=20, nstart=100, perturbation.rate=0.2, plot=F)
#save(file="London_cycles_dynsbm.Rdata",res_dynsbm,data.day1)


##############
#### Compare results
load("London_cycles_dynsbm.Rdata")
#str(res_dynsbm[[1]])

# Model selection 
ll <- NULL
ICL <- NULL
for (q in Qmin:Qmax){
  ll <- c(ll,res_dynsbm[[q]]$loglikelihood)
  # T-1 = 24-1 = 23
  ICL.q <- res_dynsbm[[q]]$loglikelihood -q*(q-1)/2*log(n1*23) -q/2*log(n1*(n1-1)*24/2) - q*(q-1)/4*24*log(n1*(n1-1)/2) 
  ICL <- c(ICL,ICL.q)
}
ICL
plot(seq(Qmin,Qmax),ll,type="l",xlab="Q")
lines(Qmin:Qmax,ICL,type="l",col="red")
# their model selects Q=4 groups 

# analysis with 4 groups 
res_dynsbm4 <- res_dynsbm[[4]]
apply(res_dynsbm4$membership,2,table)
alluvial.plot(res_dynsbm4)

# Behaviour of the clusters
apply(res_dynsbm4$membership==1,2,sum)
apply(res_dynsbm4$membership==2,2,sum)
apply(res_dynsbm4$membership==3,2,sum)
apply(res_dynsbm4$membership==4,2,sum)
# summaries of these numlber of stations per groups
summary(apply(res_dynsbm4$membership==1,2,sum))
summary(apply(res_dynsbm4$membership==2,2,sum))
summary(apply(res_dynsbm4$membership==3,2,sum))
summary(apply(res_dynsbm4$membership==4,2,sum))

# Clusters for the 2 special stations number 14 and 154
res_dynsbm4$membership[14,]
# this station sometimes appears in the small groups 2/3
res_dynsbm4$membership[154,]
# this station is clustered with large (4) and medium size (1) group

