################################
################################
#### PRIMARY SCHOOL DATASET ANALYSIS
################################
################################


################################
#### Preprocessing of the data
################################

rm(list=ls())

#########
# Extract the data
#########
donnees <- read.table("primaryschool.csv",sep="\t",header=F) 
n= length(unique(c(donnees$V3,donnees$V2)))

# Renumbering the students from 1 to n  
toto <- c(donnees$V2,donnees$V3)
toto <- match(toto, unique(sort(toto)))
correspondance= cbind(nom=donnees$V2,indice=toto[1:length(donnees$V2)])
donnees$V2 <-toto[1:length(donnees$V2)]
donnees$V3 <- toto[(length(donnees$V2)+1):(length(donnees$V2)+length(donnees$V3))]

# Transform pair of dyads (i,j) into type and add it as V6
# recall that dyad (i,j) with i<j is encoded as (2*n-i)*(i-1)/2 +j-i 
titi<-cbind(donnees$V2,donnees$V3)
titi <- t(apply(titi,1,sort))
donnees$V6 <- (2*n-titi[,1])*(titi[,1]-1)/2 +titi[,2]-titi[,1] 


# Translate initial time 
start <- donnees$V1[1]
Time <-max(donnees$V1)-start

N <- n*(n-1)/2
obs=list(time.seq = donnees$V1- start, type.seq= donnees$V6,Time=Time)


#############
# Construct point processes statistics - with dyadics partitions 2^8 
#############
setwd("../code/")
source("auxiliary.R")
setwd("../data/")

Dmax=2^8
data = list(Time=Time,Nijk = statistics(obs,n,Dmax,directed=FALSE))
#save(list=ls(),file="primaryschool_data_Dyadique_8.Rdata")

####################################################
#### Dynppsbm analysis - non directed - non sparse
####################################################
setwd("../code/")
source("vem_dynppsbm.R")
source('output.R')
setwd("../data/")

Qmin <- 1
Qmax <- 25
res_vem <- list()
nb.coeurs <- detectCores()

## WARNING ! THIS STEP TAKES SOME TIME !
# You can go directly to loading our results
load("primaryschool_data_Dyadique_8.Rdata")
res_vem=dynppSBM(data,n,Qmin,Qmax,directed=FALSE,sparse=FALSE,method='hist',cores=nb.coeurs,l_part=6,n_perturb=20,perc_perturb=.2,n_random=0)
#save(res_vem,file="primaryschool_vem_Dmax_8_modelsel_1_25.Rdata")

#loading our results
Qmin <- 1
Qmax <- 25
#load("primaryschool_data_Dyadique_8.Rdata")
#load("primaryschool_vem_Dmax_8_modelsel_1_25.Rdata")

# ICL criterion
modelsel <- model.selection_Q(data,n,Qmin,Qmax,directed=FALSE,sparse=FALSE,sol.hist.sauv=res_vem)
pdf("primaryschool_ICL.pdf",width=6.2,height=4)
plot.modelselec.Q(modelsel)
dev.off()


Qbest <- modelsel$Qbest
# Qbest = 17



######### RESULTS ANALYSIS 
Q= Qbest
tau=round(res_vem[[Q]]$tau)


# children per group 
info=read.table("metadata_primaryschool.txt")
gr=matrix(NA,Q,length(table(info$V2))) 
rownames(gr)=1:Q
colnames(gr)=levels(info[,2])
for (q in 1:Q)
  { gr[q,]=as.vector(table(info[(1:n)[tau[q,]==1],2])) }


# Plot the clusters 
pdf("primaryschool_tau.pdf",width=6.2,height=4)
par(mar = c(3,3,3,3))
nf <- layout(matrix(1:2,2,1, byrow=T),heights=c(4,1.5))
couleurs=c("red","magenta","blue","green","yellow")
barplot(t(gr),col=c(rep(couleurs,each=2),"black"),density=rep(c(-1,30),5),main="Q=17")
par(mar = c(1,0,0.5,0))
plot(1, type = "n", axes=FALSE, xlab="", ylab="")
legend(x = "top",inset = 0,legend=levels(info[,2])[1:6],fill=c(rep(couleurs[1:3],each=2)),horiz=TRUE,density=rep(c(-1,30),3),bty="n",cex=1.2)
legend(x = "bottom",inset = 0,legend=c("4A","4B","5A","5B","T"),fill=c(rep(couleurs[4:5],each=2),"black"),density=c(rep(c(-1,30),2),-1),horiz=TRUE,bty="n",cex=1.2)
dev.off()



# intensity estimates for groups (q, 1)
T <- data$Time
loglambda.hist=res_vem[[Q]]$loglambda.ql
best.d=res_vem[[Q]]$best.d

library(ggplot2)
library(reshape2)
library(gridExtra)

intensity_ql <- function(q,l)
{ ind.ql <-  indice(q,l,Q,directed=F)
  values <- loglambda.hist[ind.ql,]
  values[loglambda.hist[ind.ql,]!=0] <- exp(loglambda.hist[ind.ql,loglambda.hist[ind.ql,]!=0] )
  data.plot<-data.frame(Time=seq(0,T,by=T/length(values)),Intensity=c(values,values[Dmax]))
  plot<- ggplot(data=data.plot,aes(x=Time,y=Intensity))+geom_line()+
    scale_x_continuous(breaks=c(4500,4500+8*3600, 4500+22*3600, 4500+32*3600),
                       labels = c("10am", "6pm","8am","6pm"))+
    labs(title =paste0("(q,l)=(",q,",",l,")")) +
    theme(plot.title = element_text(hjust = 0.5)) 
  return(plot)
}

# a plot with more values on real axis
intensity_ql_bis <- function(q,l)#,yscale=0.002)
{ ind.ql <-  indice(q,l,Q,directed=F)
values <- loglambda.hist[ind.ql,]
values[loglambda.hist[ind.ql,]!=0] <- exp(loglambda.hist[ind.ql,loglambda.hist[ind.ql,]!=0] )
data.plot<-data.frame(Time=seq(0,T,by=T/length(values)),Intensity=c(values,values[Dmax]))
plot<- ggplot(data=data.plot,aes(x=Time,y=Intensity))+geom_line()+
  scale_x_continuous(breaks=c(4500,4500+4*3600,4500+6*3600,4500+8*3600, 4500+22*3600,4500+26*3600,4500+28*3600, 4500+30*3600),
                     labels = c("10am", "2pm","4pm","6pm","8am","10am","2pm","4pm"))+
  labs(title =paste0("(q,l)=(",q,",",l,")")) +
  theme(plot.title = element_text(hjust = 0.5)) 
return(plot)
}
 

# intra group (except group 11 discussed later)
plot.list<-list()
i <-0
for (q in 1:17){  
  l=q
  if ((sum(tau[q,]==1))>2){
    i <- i+1
    plot.list[[i]]<- intensity_ql(q, l)#,yscale=0.005)
  } 
}
final.plot <- do.call("grid.arrange",c(plot.list, ncol=3,nrow=4))
ggsave("primaryschool_intra_group.pdf",final.plot,width =6,height=8)


#intra-group intensity : examples of group 9 (high intra-group intensity) versus group 6(lowest intra-group intensity)
#pdf("primaryschool_groups_9versus6.pdf",width=6.2,height=4)
#par(mfrow=c(1,2))
#intensity_ql(q=9, l=9)#,yscale=0.002)
#intensity_ql(q=6, l=6)#,yscale=0.005)
#dev.off()

# two classes of the same grade
psame <- intensity_ql_bis( q=5, l=13)
ggsave("primaryschool_samegrade.pdf",psame,width =5.8,height=3)



# specific behavior of children in class 2B
p1 <- intensity_ql(q=11, l=11)
p2 <- intensity_ql(q=11, l=12)
p3 <- intensity_ql(q=11, l=17)
p4 <- intensity_ql(q=12, l=17)
p.global <- grid.arrange(p1,p2,p3,p4, nrow=2,ncol=2)
ggsave("primaryschool_class_2B.pdf",p.global,width =5.8,height=3)


# class 1B splitted into groups 1 and 16
#pdf("primaryschool_class_1B.pdf",width=6.2,height=4)
#par(mfrow=c(1,3))
#intensity_ql(q=16, l=16,yscale=0.005)
#intensity_ql( q=1, l=1,yscale=0.005)
#intensity_ql(q=1, l=16,yscale=0.005)
#dev.off()


####################################################
#### Dynppsbm analysis - non directed - sparse
####################################################
mean(rowSums(data$Nijk)>0)
#0.2852097
#28% of pairs of individuals have at least one interaction


setwd("../code/")
source("vem_dynppsbm.R")
source('output.R')
setwd("../data/")

Qmin <- 1
Qmax <- 15
res_vem <- list()
nb.coeurs <- detectCores()

## WARNING ! THIS STEP TAKES SOME TIME !
# You can go directly to loading our results
load("primaryschool_data_Dyadique_8.Rdata")
res_vem=dynppSBM(data,n,Qmin,Qmax,directed=FALSE,sparse=TRUE,method='hist',cores=nb.coeurs,l_part=3,n_perturb=20,perc_perturb=.2,n_random=0)
#save(res_vem,file="primaryschool_vem_Dmax_8_modelsel_1_15_sparse.Rdata")


#loading our results
Qmin <- 1
Qmax <- 15
load("primaryschool_data_Dyadique_8.Rdata")
load("primaryschool_vem_Dmax_8_modelsel_1_15_sparse.Rdata")


# ICL criterion
modelsel <- model.selection_Q(data,n,Qmin,Qmax,directed=FALSE,sparse=TRUE,sol.hist.sauv=res_vem)
plot.modelselec.Q(modelsel)
Qbest <- modelsel$Qbest
# Qbest = 13


######### RESULTS ANALYSIS 
Q= Qbest
tau=round(res_vem[[Q]]$tau)


# children per group 
info=read.table("metadata_primaryschool.txt")
gr=matrix(NA,Q,length(table(info$V2))) 
rownames(gr)=1:Q
colnames(gr)=levels(info[,2])
for (q in 1:Q)
  { gr[q,]=as.vector(table(info[(1:n)[tau[q,]==1],2])) }


# Plot the clusters 
par(mar = c(3,3,3,3))
nf <- layout(matrix(1:2,2,1, byrow=T),heights=c(4,1.5))
couleurs=c("red","magenta","blue","green","yellow")
barplot(t(gr),col=c(rep(couleurs,each=2),"black"),density=rep(c(-1,30),5),main="Q=17")
par(mar = c(1,0,0.5,0))
plot(1, type = "n", axes=FALSE, xlab="", ylab="")
legend(x = "top",inset = 0,legend=levels(info[,2])[1:6],fill=c(rep(couleurs[1:3],each=2)),horiz=TRUE,density=rep(c(-1,30),3),bty="n",cex=1.2)
legend(x = "bottom",inset = 0,legend=c("4A","4B","5A","5B","T"),fill=c(rep(couleurs[4:5],each=2),"black"),density=c(rep(c(-1,30),2),-1),horiz=TRUE,bty="n",cex=1.2)



# intensity
T <- data$Time
loglambda.hist=res_vem[[Q]]$loglambda.ql
best.d=res_vem[[Q]]$best.d


# intra group intensity
plot.list<-list()
i <- 0
for (q in 1:13)
{ l=q
i <- i+1
  plot.list[[i]]<- intensity_ql(q, l)#,yscale=0.002)
}
final.plot <- do.call("grid.arrange",c(plot.list, ncol=4,nrow=4))


# specific behavior of children in class 4B et 1A (groups 1 and 3)
p1<- intensity_ql(q=1, l=1)
p2 <- intensity_ql(q=3, l=3)
pc<- grid.arrange(p1,p2,ncol=2)
ggsave("primaryschool_sparse_class_4B_1A.pdf",pc,width =5.8,height=3)


# estimated intra-connectivity probability
beta_11 = res_vem[[Q]]$beta[indice(1,1,12,directed=F)]
beta_33 = res_vem[[Q]]$beta[indice(3,3,12,directed=F)]
#> beta_11
#[1] 0.8404548
#> beta_33
#[1] 0.2954211


