####################################################
#### Functions to generate data under Dynppsbm
####################################################



#############
# generate realizations of an inhomogeneous PP
#############

# intens: function defined on [0,Time]
# max.intens : upper bound of intens on [0,Time]
generat.intens <- function(intens, Time, max.intens){
  area <- Time*max.intens
  M <- rpois(1,area)
  candid.points <- runif(M,0,Time)
  prob <- intens(candid.points)/max.intens
  thinning <- rbinom(rep(1,M),1,prob)
  points <- candid.points[thinning==1]
  return(points)	
}

#############
# generate data under dynppsbm 
#############
# intens: list of intensities alpha^(q,l)
# prop.groups: vector of group proportions, should be of length Q
# if directed=TRUE then intens should be of length Q^2 and if directed =FALSE then length Q*(Q+1)/2

generat.dynppSBM <- function(intens,Time,n,prop.groups,directed=TRUE){
  Q <- length(prop.groups)
  
  # test coherence
  if (directed){
    if (length(intens)!=Q^2){ stop("not a correct number of intensities")}
  }else{ if (length(intens)!=Q*(Q+1)/2){stop("not a correct number of intensities")}}
  
  if (directed){ 
    N <- n*(n-1)
    } else{N <- n*(n-1)/2}
  
  # Draw hidden groups
  z <- rmultinom(n, 1, prop.groups)
  group <- apply(z,2,which.max)
  
  # Generate the process
  time.seq <- NULL
  type.seq <- NULL
  if (directed){ # directed
    for (i in 1:n)
      for (j in (1:n)[-i]){
        type.ql <- indice(group[i],group[j],Q,directed)
        proc <- generat.intens(intens[[type.ql]]$intens,Time,intens[[type.ql]]$max)
        if (length(proc)>0){
          time.seq <- c(time.seq,proc)  	# not ordered 	
          type.seq <- c(type.seq,rep(which.dyads(i,j,n,directed),length(proc)))  			
        }
      }
  }else{ # undirected
    for (i in 1:(n-1))
      for (j in (i+1):n){
        type.ql <- indice(group[i],group[j],Q,directed)
        proc <- generat.intens(intens[[type.ql]]$intens,Time,intens[[type.ql]]$max)
        if (length(proc)>0){
          time.seq <- c(time.seq,proc)  	# not ordered 
          type.seq <- c(type.seq,rep(which.dyads(i,j,n,directed),length(proc)))  			
        }
      }
    }
  ordre <- order(time.seq)
  time.seq <- time.seq[ordre]
  type.seq <- type.seq[ordre]
  data <-	list(time.seq=time.seq,type.seq=type.seq,Time=Time)
  
  return(list(data=data,z=z,intens=intens))
}


