##############################
### Simulations second scenario - Risks and ARI
##############################
rm(list=ls())
source('generator.R')
#setwd("../code")
source('vem_dynppsbm.R')
source("output.R")
#setwd("../simulations/")


###################
## calculation of the risk
###################
# structures of kernel and hist estimates are different ! 

risk <- function(intens,lambda.kernel,log.lambda.hist,Q=3,step=0.001){ 
  risk.ker <- rep(NA,Q*(Q+1)/2)
  risk.hist <- rep(NA,Q*(Q+1)/2)
  x0 <- seq(0,1,by=step)
  Dmax <- dim(log.lambda.hist)[2]
  
  ind.ql <- 0
  for (q in 1:Q)
    for (l in q:Q){
      ind.ql <- ind.ql + 1
      true.val <- intens[[ind.ql]]$intens(x0)	
      risk.ker[ind.ql] <- sqrt(sum((true.val-lambda.kernel[ind.ql,])^2*step))
      lambda.hist.vec <- rep(NA,length(true.val))		
      for (s in 1:Dmax){
        down <- (s-1)/Dmax
        down <- ceiling(down/step) + 1
        up <- s/Dmax
        up <- floor(up/step) + 1
        lambda.hist.vec[down:up] <- exp(log.lambda.hist[ind.ql,s])				
      }
      risk.hist[ind.ql] <- sqrt(sum((true.val-lambda.hist.vec)^2*step))
    }
  return(list(risk.hist=risk.hist, risk.ker=risk.ker))
}



###################
## compute the numbers of points per color process
###################

nb.points.per.color <- function(obs,n,Q=3){
  z <- apply(obs$z,2,which.max)
  nb.points.ql <- rep(0,Q*(Q+1)/2)
  ind.ij <- 0
  for (i in 1:(n-1))
    for (j in (i+1):n){
      ind.ij <- ind.ij +1
      color.ql <- indice(z[i],z[j],Q,directed=F)
      nb.points.ql[color.ql] <- nb.points.ql[color.ql] + length(obs$data$time.seq[obs$data$type.seq== ind.ij])
    }
  return(nb.points.ql)
}

###################
## intensity functions
###################

constante <- function(x) return (rep(4,length(x)))

piece.const <- function(x){
  y <- rep(0,length(x))
  y[x<.25] <- 4
  y[x>.75] <- 10
  return(y)
}

triang <- function(x) return(8*(1/2-abs(x-1/2)))

xexp <- function(x)
  return(100*x*exp(-8*x))

expsin <- function(x)
  return(exp(3*x)*(sin(6*pi*x-pi/2)+1)/2)

dblexp <- function(x)
  return(8.1*(exp(-6*abs(x-1/2))-.049))	

Q <- 3
prop.groups <- rep(1/Q,Q)
Time <- 1
step <- .001	
x0 <- seq(0,Time,by=step)

intens <- list(NULL)
intens[[1]] <- list(intens= constante,max=max(constante(x0))+.1) 	# (q,l) = (1,1)  
intens[[2]] <- list(intens= piece.const,max=max(piece.const(x0))+.1)	# (q,l) = (1,2)  
intens[[3]] <- list(intens= triang,max=max(triang(x0))+.1) 	# (q,l) = (1,3)  
intens[[4]] <- list(intens= xexp,max=max(xexp(x0))+.1) 	# (q,l) = (2,2)  
intens[[5]] <- list(intens= expsin,max=max(expsin(x0))+.1) 	# (q,l) = (2,3)
intens[[6]] <- list(intens= dblexp,max=max(dblexp(x0))+.1) 	# (q,l) = (3,3)  


#################################
## repeated simulations 
#################################
REP <- 1000
Dmax <- 2^3

# Choose value of n here
#n <- 20
n <- 50
# Warning: do not use multiple cores with set.seed

risk.ker <- matrix(NA,REP,Q*(Q+1)/2)
risk.hist <- matrix(NA,REP,Q*(Q+1)/2)
risk.ker.oracle <- matrix(NA,REP,Q*(Q+1)/2)
risk.hist.oracle <- matrix(NA,REP,Q*(Q+1)/2)
res.ARI <- matrix(NA,REP,2) # 1st column : kernel, 2nd column : hist
nb.points.ql <- matrix(NA,REP,Q*(Q+1)/2)


for (rep in 1:REP){	
  cat('************* repetition', rep, '*************\n')
  # generate data
  set.seed(rep)
  obs <- generat.dynppSBM(intens,Time,n,prop.groups,directed=F)	
  obs$Nijk <- statistics(obs$data,n,Dmax,directed=F)
  nb.points.ql[rep,] <- nb.points.per.color(obs,n)
  
  ### Oracle estimates
  log.lambda.hist.oracle <- Mstep(list(Nijk=obs$Nijk,Time=obs$data$Time),list(tau=obs$z,rho=NULL),directed=F,sparse=F)$loglambda.ql
  lambda.kernel.oracle <- hat.lambda.fn(obs$data,obs$z,Q,n,directed=FALSE) 
  
  # compute both estimators
  set.seed(rep) # same intialization for both methods
  sol.hist <- dynppSBM(list(Nijk=obs$Nijk,Time=obs$data$Time),n,Q,directed=F,method='hist',l_part=4,n_perturb=0,n_random=0)[[1]]
  # default values sparse=FALSE,perc_perturb=.2,nb.iter=50,fix.iter=10,epsilon=1e-6
  log.lambda.hist <- permut.lambda(sol.hist$loglambda.ql,obs$z,sol.hist$tau,directed=F)
  
  set.seed(rep) # same intialization for both methods
  sol.kernel <- dynppSBM(obs$data,n,Q,directed=F,method='kernel',l_part=4,n_perturb=0,n_random=0)[[1]]
  # default values sparse=FALSE,perc_perturb=.2,nb.iter=50,fix.iter=10,epsilon=1e-6
  sol.kernel.lambda <- hat.lambda.fn(obs$data,sol.kernel$tau,Q,n,directed=F)
  lambda.kernel <- permut.lambda(sol.kernel.lambda,obs$z,sol.kernel$tau,directed=F)
  
  # risks
  res.risk.oracle <- risk(intens,lambda.kernel.oracle,log.lambda.hist.oracle)
  res.risk <- risk(intens,lambda.kernel,log.lambda.hist)
  risk.ker[rep,] <- res.risk$risk.ker
  risk.hist[rep,] <- res.risk$risk.hist
  risk.ker.oracle[rep,] <- res.risk.oracle$risk.ker
  risk.hist.oracle[rep,] <- res.risk.oracle$risk.hist
  
  # ARI
  res.ARI[rep,2] <- ARI(obs$z,sol.hist$tau)
  res.ARI[rep,1] <- ARI(obs$z,sol.kernel$tau)
}

# draw figures for the last dataset
library(ggplot2)
library(gridExtra)
library("reshape2")
g_legend<-function(a.gplot){
  tmp <- ggplot_gtable(ggplot_build(a.gplot))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)}

ind.ql <- 0
plot.list <-list()
for (q in 1:Q)
  for (l in q:Q){
    ind.ql <- ind.ql + 1
    true.val <- intens[[ind.ql]]$intens(x0)
    nb.replicates <- c(126,rep(125,7)) # valid with Dmax=2^3
    data.plot <- data.frame(Time=x0,alpha=true.val,hat.alpha.ker=lambda.kernel[ind.ql,],hat.alpha.hist=rep(exp(log.lambda.hist[ind.ql,]),nb.replicates))
    data.plot.long <- melt(data.plot, id="Time")  # convert to long format
    plot.list[[ind.ql]] <- ggplot(data.plot.long, aes(x=Time,y=value,linetype=variable))+ 
      geom_line() + 
      labs(title =paste0("(q,l)=(",q,",",l,")"),y = "Intensities")  +
      #ggtitle(paste0("(q,l)=(",q,",",l,")")) +
      theme(plot.title = element_text(hjust = 0.5),legend.position="none") 
  }
# fake plot for legend
true.val <- intens[[ind.ql]]$intens(x0)
nb.replicates <- c(126,rep(125,7)) # valid with Dmax=2^3
data.plot <- data.frame(Time=x0,alpha=true.val,hat.alpha.ker=lambda.kernel[ind.ql,],hat.alpha.hist=rep(exp(log.lambda.hist[ind.ql,]),nb.replicates))
data.plot.long <- melt(data.plot, id="Time")  # convert to long format
fake.plot <- ggplot(data.plot.long, aes(x=Time,y=value,linetype=variable))+ 
  geom_line() + 
  labs(linetype = "")  +
  theme(plot.title = element_text(hjust = 0.5),legend.position="bottom") 
##
mylegend <- g_legend(fake.plot)
interm.plot <- do.call("grid.arrange",c(plot.list, ncol=3,nrow=2))
final.plot <- grid.arrange(interm.plot, mylegend, nrow=2,heights=c(10,1))
file=paste("fig_intensities_Q3_n",n,".pdf",sep="")
ggsave(file, plot=final.plot,width = 5.8,height=3)


# save results
results <- cbind(nb.points.ql,risk.ker,risk.hist,risk.ker.oracle,risk.hist.oracle,res.ARI)
colnames(results)=c(paste("nb.points.ql",1:(Q*(Q+1)/2)),paste("risk.ker",1:(Q*(Q+1)/2)),paste("risk.hist",1:(Q*(Q+1)/2)),paste("risk.or.ker",1:(Q*(Q+1)/2)),paste("risk.or.hist",1:(Q*(Q+1)/2)),"ARI.ker","ARI.hist")
write.table(results, file=paste('results-risk-with-oracle-n-',n,'-rep-',REP,'.txt',sep=""))

file=paste('results_scenar2_n',n,'.Rdata',sep="")
save(n,REP,Dmax,risk.ker,risk.hist,risk.ker.oracle,risk.hist.oracle,res.ARI,nb.points.ql,lambda.hist,lambda.kernel,lambda.hist.oracle,lambda.kernel.oracle,file=file)


################################
### Results Outputs
################################
### Create Latex table
risk20 <- read.table("results-risk-with-oracle-n-20-rep-1000.txt",header=TRUE)
risk50 <- read.table("results-risk-with-oracle-n-50-rep-1000.txt",header=TRUE)
moy20 <- apply(risk20,2,mean)
sd20 <- apply(risk20,2,sd)
moy50 <- apply(risk50,2,mean)
sd50 <- apply(risk50,2,sd)
library('xtable')
tab20 <- data.frame(NB.events=round(moy20[1:6]),Hist=round(100*moy20[13:18]),Hist.sd=round(100*sd20[13:18]),Or.Hist=round(100*moy20[25:30]),Or.Hist.sd=round(100*sd20[25:30]),Ker=round(100*moy20[7:12]),Ker.sd=round(100*sd20[7:12]),Or.Ker=round(100*moy20[19:24]),Or.Ker.sd=round(100*sd20[19:24]))
xtable(tab20,digits=0)

tab50 <- data.frame(NB.events=round(moy50[1:6]),Hist=round(100*moy50[13:18]),Hist.sd=round(100*sd50[13:18]),Or.Hist=round(100*moy50[25:30]),Or.Hist.sd=round(100*sd50[25:30]),Ker=round(100*moy50[7:12]),Ker.sd=round(100*sd50[7:12]),Or.Ker=round(100*moy50[19:24]),Or.Ker.sd=round(100*sd50[19:24]))
xtable(tab50,digits=0)

### Output ARI boxplots
# str(risk20)
method <- factor(rep(c(1,2),c(1000,1000)),labels=c('hist','ker'))
ARI20 <- data.frame(ARI=c(risk20$ARI.hist,risk20$ARI.ker),method=method)
ARI50 <- data.frame(ARI=c(risk50$ARI.hist,risk50$ARI.ker),method=method)
plot20 <-ggplot(data = ARI20, aes(x=" ",y =ARI,fill=method,dodge=method)) + geom_boxplot() +
  xlab(label="") +scale_fill_grey(start=0.5, end=1) +  theme(legend.position="bottom") + 
  ggtitle("n=20") +theme(plot.title = element_text(hjust = 0.5))

plot50 <-ggplot(data = ARI50, aes(x=" ",y =ARI,fill=method,dodge=method)) + geom_boxplot() +
  xlab(label="") +scale_fill_grey(start=0.5, end=1) +  theme(legend.position="bottom") + 
  ggtitle("n=50") +theme(plot.title = element_text(hjust = 0.5))
mylegend<-g_legend(plot20)
final.plot <- grid.arrange(arrangeGrob(plot20 + theme(legend.position="none"), plot50 + theme(legend.position="none"), nrow=1),mylegend, nrow=2,heights=c(10,1))
ggsave("boxplot_Q3_n20_n50.pdf",plot=final.plot, width = 6.2,height=3)
