##############################
### Simulations second scenario - Model Selection
##############################
rm(list=ls())
source('generator.R')
#setwd("../code")
source('vem_dynppsbm.R')
source("output.R")
#setwd("../simulations/")

library(ggplot2)
library(gridExtra)

###################
## intensity functions
###################

constante <- function(x) return (rep(4,length(x)))

piece.const <- function(x){
  y <- rep(0,length(x))
  y[x<.25] <- 4
  y[x>.75] <- 10
  return(y)
}

triang <- function(x) return(8*(1/2-abs(x-1/2)))

xexp <- function(x)
  return(100*x*exp(-8*x))

expsin <- function(x)
  return(exp(3*x)*(sin(6*pi*x-pi/2)+1)/2)

dblexp <- function(x)
  return(8.1*(exp(-6*abs(x-1/2))-.049))	

Q <- 3
prop.groups <- rep(1/Q,Q)
Time <- 1
step <- .001	
x0 <- seq(0,Time,by=step)

intens <- list(NULL)
intens[[1]] <- list(intens= constante,max=max(constante(x0))+.1) 	# (q,l) = (1,1)  
intens[[2]] <- list(intens= piece.const,max=max(piece.const(x0))+.1)	# (q,l) = (1,2)  
intens[[3]] <- list(intens= triang,max=max(triang(x0))+.1) 	# (q,l) = (1,3)  
intens[[4]] <- list(intens= xexp,max=max(xexp(x0))+.1) 	# (q,l) = (2,2)  
intens[[5]] <- list(intens= expsin,max=max(expsin(x0))+.1) 	# (q,l) = (2,3)
intens[[6]] <- list(intens= dblexp,max=max(dblexp(x0))+.1) 	# (q,l) = (3,3)  


#################################
## repeated simulations 
#################################
# Settings 
# Warning: do not use multiple cores with set.seed
REP <- 1000
Dmax <- 2^3 
Q <- 3
prop.groups <- rep(1/Q,Q)
Time <- 1
Qmin <- 1
Qmax <- 10
# Select value of n here
n <- 20  
#n <- 50


Qbest=rep(NA,REP)
ARI_Qstar=rep(NA,REP)

for (rep in 1:REP){	
  cat('************* repetition', rep, '*************\n')
  # generate data
  set.seed(rep)
  obs <- generat.dynppSBM(intens,Time,n,prop.groups,directed=F)	
  obs$Nijk <- statistics(obs$data,n,Dmax,directed=F)
  data <- list(Nijk=obs$Nijk,Time=obs$data$Time)
  
  hist.estim.list <- dynppSBM(data,n,Qmin,Qmax,directed=F,method='hist',l_part=4,n_perturb=10,n_random=0)
  # default values sparse=FALSE,perc_perturb=.2,nb.iter=50,fix.iter=10,epsilon=1e-6
  # ARI between true classif and estimated one with Q=3 
  ARI_Qstar[rep]= ARI(obs$z,hist.estim.list[[Q]]$tau)  
  best <- model.selection_Q(data,n,Qmin,Qmax,directed=F,sparse=FALSE,hist.estim.list)
  Qbest[rep] <- best$Qbest
}


# save results
results = rbind(Qbest,ARI_Qstar)
write.table(results, file=paste('results-scen2-Qbest-n-',n,'-rep-',REP,'.txt',sep=""))



###################
## Outputs
###################
# n= 20
res <- read.table("results-scen2-Qbest-n-20-rep-1000.txt")
apply(res,1,mean)
group <- as.integer(res[1,])
table(group)
ARI <- as.numeric(res[2,])
str(ARI)


t20<-data.frame(table(group)/REP)
plot1 <-ggplot(data = t20, aes(x=group,y =Freq)) + geom_bar(stat="identity") +
  labs(y="Frequency",x="Selected number of groups") +
  ggtitle("n=20") +theme(plot.title = element_text(hjust = 0.5))
plot2 <- ggplot(data=data.frame(ARI=ARI,group=as.factor(group)),aes(x=group,y=ARI)) +geom_boxplot() +
  labs(x="Selected number of groups",y="ARI with 3 groups")+
  ggtitle("n=20") +theme(plot.title = element_text(hjust = 0.5))
plot <- grid.arrange(plot1,plot2,ncol=2)
ggsave("Qbest-20.pdf",plot=plot, width = 6.2,height=3)


# n= 50
res <- read.table("results-scen2-Qbest-n-50-rep-1000.txt")
apply(res,1,mean)
group <- as.integer(res[1,])
table(group)
ARI <- as.numeric(res[2,])
ARI[group==4]
